use strict;
use warnings;

my $context;
my $walk_table = {
  0 => sub { $context->{ x }--; },
  1 => sub { $context->{ y }--; },
  2 => sub { $context->{ x }++; },
  3 => sub { $context->{ y }++; }
};

sub do_action
{
  my ( $action ) = @_;

  if( $action eq 'W' )
  {
    $context->{ status }->{ $context->{ x } }->{ $context->{ y } }->{ $context->{ direction } } = 1;
    $walk_table->{ $context->{ direction } }->();
  }
  elsif( $action eq 'R' )
  {
    $context->{ direction } = ( $context->{ direction } + 1 ) % 4;
  }
  elsif( $action eq 'L' )
  {
    $context->{ direction } = ( $context->{ direction } - 1 ) % 4;
  }
  elsif( $action eq 'RR' )
  {
    $context->{ direction } = ( $context->{ direction } + 2 ) % 4;
  }
}

sub process_sequence
{
  my ( $sequence ) = @_;

  my @sequence = split( "", $sequence );
  while( my $action = shift( @sequence ) )
  {
    if( $action eq 'R' )
    {
      my $next_action = shift( @sequence );

      if( $next_action eq 'R' )
      {
        do_action( 'RR' );
        next;
      }

      unshift( @sequence, $next_action );
    }

    do_action( $action );

    if( $action eq 'W' and scalar( @sequence ) != 0 )
    {
      $context->{ max_x } = $context->{ x } if( $context->{ x } > $context->{ max_x } );
      $context->{ min_x } = $context->{ x } if( $context->{ x } < $context->{ min_x } );
      $context->{ max_y } = $context->{ y } if( $context->{ y } > $context->{ max_y } );
      $context->{ min_y } = $context->{ y } if( $context->{ y } < $context->{ min_y } );
    }
  }
}

# NESW - 0123
open( INPUT, '<', 'sample_input' );
  my $total_count = <INPUT>;
  my $case_count  = 1;

  while( $case_count <= $total_count )
  {
    printf( "Case #%d:\n", $case_count++ );

    my ( $entr_to_exit, $exit_to_entr ) = split( ' ', <INPUT> );

    $context = { x => -1, y => 0, min_x => 0, min_y => 0, max_x => 0, max_y => 0, direction => 2, status => {} };
    process_sequence( $entr_to_exit );
    $context->{ direction } = ( $context->{ direction } + 2 ) % 4;
    process_sequence( $exit_to_entr );

    for my $x ( $context->{ min_x }..$context->{ max_x } )
    {
      for my $y ( reverse( $context->{ min_y }..$context->{ max_y } ) )
      {
        my $arbiter = get_arbiter( $x, $y );

        my $status_num = 8 * $arbiter->( 1 ) + 4 * $arbiter->( 3 ) + 2 * $arbiter->( 2 ) + 1 * $arbiter->( 0 );
        printf( "%x", $status_num );
      }
      print( "\n" );
    }
  }
close( INPUT );

sub get_arbiter
{
  my ( $x, $y ) = @_;
  my $cur_status = $context->{ status }->{ $x }->{ $y };

  return sub { my ( $direction ) = @_; ( $cur_status->{ $direction } ) ? return 1 : return 0; }
}
