use strict;
use warnings;

use bignum;

sub Cr
{
  my ( $n, $m ) = @_;

  $m = $m > ( $n - $m ) ? ( $n - $m ) : $m;
  return 1 if( $m == 0 );

  my $numerator = 1;
  my $counter = $n - $m + 1;
  while( $counter <= $n )
  {
    $numerator *= $counter++;
  }

  my $denominator = 1;
  $counter = 2;
  while( $counter <= $m )
  {
    $denominator *= $counter++;
  }

  return $numerator / $denominator;
}

sub get_F_max
{
  my ( $D, $B ) = @_;

  my $result;

  if( $B > $D )
  {
    $result = get_F_max( $D, $D );
  }
  elsif( $D == $B )
  {
    $result = 2 ** $D - 1;
  }
  else
  {
    $result = $D;

    if( $B > 1 )
    {
      my $counter = $D - 2;
      while( $counter >= ( $D - $B ) )
      {
        $result += Cr( $D, $counter-- );
        last if( $result > 2 ** 32 );
      }
    }
  }

  if( $result > 2 ** 32 )
  {
    return inf;
  }
  else
  {
    return $result;
  }
}

sub get_D_min
{
  my ( $F, $B ) = @_;

  my ( $prev, $curr ) = ( 1, 1 );

  while( 1 )
  {
    my $cur_F = get_F_max( $curr, $B );

    if( $cur_F > $F )
    {
      return dmin_binary_search( $F, $B, $prev, $curr );
    }
    elsif( $cur_F == $F )
    {
      return $curr;
    }
    else
    {
      $prev =  $curr;
      $curr *= 2;
    }
  }
}

sub dmin_binary_search
{
  my ( $F, $B, $low, $high ) = @_;

  my $mid = int( ( $high + $low ) / 2 );
  my $cur_F = get_F_max( $mid, $B );

  if( $cur_F > $F )
  {
    return $mid if( $mid == $low + 1 );
    return dmin_binary_search( $F, $B, $low, $mid );
  }
  elsif( $cur_F == $F )
  {
    return $mid;
  }
  else
  {
    return $high if( $high == $mid + 1 );
    return dmin_binary_search( $F, $B, $mid, $high );
  }
}

sub get_B_min
{
  my ( $F, $D ) = @_;

  my $B = 1;
  while( 1 )
  {
    my $cur_F = get_F_max( $D, $B );

    return $B  if( $cur_F >= $F );

    $B++;
  }
}

sub normalize { my ( $val ) = @_; ( $val == inf ) ? return -1 : return $val; }

open( INPUT, '<', 'sample_input' );
  my $total_count = <INPUT>;
  my $case_count  = 1;

  my $F_max_cache = { 0-0 => 0 };
  while( $case_count <= $total_count )
  {
    my ( $F, $D, $B ) = split( ' ', <INPUT> );

    printf( "Case #%d: %.0f %d %d\n", $case_count++, normalize( get_F_max( $D, $B ) ), normalize( get_D_min( $F, $B ) ), normalize( get_B_min( $F, $D ) ) );
  }
close( INPUT );
