#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

#define INFINITE DBL_MAX

#define MAX_READ_BUF_SIZE 10
int read_int( void )
{
  char buf[ MAX_READ_BUF_SIZE ];
  if( NULL == fgets( buf, MAX_READ_BUF_SIZE, stdin ) )
    exit( -1 );
  return atoi( buf );
}

int g_case_count = 0;

int c_item_count      = 0;
int c_store_count     = 0;
int c_gas_price       = 0;
int c_perishable_mask = 0;

#define MAX_STORE_COUNT 50
#define HOME MAX_STORE_COUNT
#define MAX_ITEM_COUNT  15
int c_store_x[ MAX_STORE_COUNT ];
int c_store_y[ MAX_STORE_COUNT ];
int c_store_item_mask[ MAX_STORE_COUNT ];
int c_store_item_price[ MAX_STORE_COUNT ][ MAX_ITEM_COUNT ];

double calc_transport_fee( int place_1, int place_2 )
{
  int x1 = ( place_1 == HOME ? 0 : c_store_x[ place_1 ] );
  int y1 = ( place_1 == HOME ? 0 : c_store_y[ place_1 ] );
  int x2 = ( place_2 == HOME ? 0 : c_store_x[ place_2 ] );
  int y2 = ( place_2 == HOME ? 0 : c_store_y[ place_2 ] );

  int xdiff = x2 - x1;
  int ydiff = y2 - y1;

  return c_gas_price * sqrt( xdiff * xdiff + ydiff * ydiff );
}
double c_transport_fee_cache[ MAX_STORE_COUNT + 1 ][ MAX_STORE_COUNT + 1 ];
inline double transport_fee( int place_1, int place_2 )
{
  if( c_transport_fee_cache[ place_1 ][ place_2 ] != -1 )
    return c_transport_fee_cache[ place_1 ][ place_2 ];
  else
    return c_transport_fee_cache[ place_1 ][ place_2 ]
      = c_transport_fee_cache[ place_2 ][ place_1 ]
      = calc_transport_fee( place_1, place_2 );
}

double c_get_min_cost_cache[ MAX_STORE_COUNT ][ 1 << MAX_ITEM_COUNT ][ 2 ];
double get_min_cost( int cur_store, int shopping_list, int decide_to_go_home )
{
  if( c_get_min_cost_cache[ cur_store ][ shopping_list ][ decide_to_go_home ] != -1 )
    return c_get_min_cost_cache[ cur_store ][shopping_list][ decide_to_go_home ];

  double result = INFINITE;

  int item_checker = c_store_item_mask[ cur_store ] & shopping_list;
  int item_cnt = 0;
  for( ; item_cnt < c_item_count; item_cnt++ )
  {
    if( ! ( item_checker & ( 1 << item_cnt ) ) )
      continue;

    int next_shopping_list = shopping_list ^ ( 1 << item_cnt );

    if( next_shopping_list == 0 )
    {
      double temp_result = c_store_item_price[ cur_store ][ item_cnt ] + transport_fee( cur_store, HOME );

      if( temp_result < result )
        result = temp_result;
    }
    else
    {
      int go_home = ( decide_to_go_home || ( c_perishable_mask & ( 1 << item_cnt ) ) );

      int next_store = 0;
      for( ; next_store < c_store_count; next_store++ )
      {
        if( ! ( next_shopping_list & c_store_item_mask[ next_store ] ) )
          continue;

        if( next_store == cur_store )
        {
          double temp_result = c_store_item_price[ cur_store ][ item_cnt ] + get_min_cost( next_store, next_shopping_list, go_home );

          if( temp_result < result )
            result = temp_result;
        }
        else
        {
          double temp_result = c_store_item_price[ cur_store ][ item_cnt ];

          if( go_home )
            temp_result += transport_fee( cur_store, HOME ) + transport_fee( HOME, next_store );
          else
            temp_result += transport_fee( cur_store, next_store );

          temp_result += get_min_cost( next_store, next_shopping_list, 0 );

          if( temp_result < result )
            result = temp_result;
        }
      }
    }
  }

  return c_get_min_cost_cache[ cur_store ][ shopping_list ][ decide_to_go_home ] = result;
}

double run_case( void )
{
  double result = INFINITE;

  int start_store = 0;
  for( ; start_store < c_store_count; start_store++ )
  {
    double temp_result = transport_fee( HOME, start_store )
      + get_min_cost( start_store, ( 1 << c_item_count  ) - 1, 0 );
    if( temp_result < result )
      result = temp_result;
  }

  return result;
}

int main( void )
{
  g_case_count = read_int();

  int case_cnt = 1;
  for( ; case_cnt <= g_case_count; case_cnt++ )
  {
    c_item_count      = read_int();
    c_store_count     = read_int();
    c_gas_price       = read_int();
    c_perishable_mask = read_int();

    int store_cnt = 0;
    for( ; store_cnt < c_store_count; store_cnt++ )
    {
      c_store_x[ store_cnt ] = read_int();
      c_store_y[ store_cnt ] = read_int();

      int store_item_cnt = read_int();
      for( ; store_item_cnt > 0; store_item_cnt-- )
      {
        int store_item_idx   = read_int();
        c_store_item_price[ store_cnt ][ store_item_idx ] = read_int();
      }
      c_store_item_mask[ store_cnt ] = read_int();
    }

    int i,j;
    for( i = 0; i < c_store_count; i++ )
    {
      c_transport_fee_cache[ HOME ][ i ] = -1;
      c_transport_fee_cache[ i ][ HOME ] = -1;

      for( j = 0; j < c_store_count; j++ )
        c_transport_fee_cache[ i ][ j ] = -1;
    }

    for( i = 0; i < c_store_count; i++ )
      for( j = 0; j < ( 1 << c_item_count ); j++ )
      {
        c_get_min_cost_cache[ i ][ j ][ 0 ] = -1;
        c_get_min_cost_cache[ i ][ j ][ 1 ] = -1;
      }

    printf( "Case #%d: %.7f\n", case_cnt, run_case() );
  }

  return 0;
}
