use strict;
use warnings;

use constant INF => 9999999999999999999;
my $HOME = -1;
my $HOME_OBJ = { x => 0, y => 0 };

sub get_min_cost
{
  my ( $shopping_list, $cur_store, $decide_to_go_home, $context ) = @_;

  my $result = INF;
  my $cur_store_items  = $context->{ stores }->[ $cur_store ]->{ items };
  my $cache  = $context->{ get_min_cost_cache };

  for my $item ( keys %{ $cur_store_items } )
  {
    next unless( $shopping_list & ( 1 << $item ) );

    my $next_shopping_list = $shopping_list ^ ( 1 << $item );

    if( $next_shopping_list == 0 )
    {
      my $trans_price = $context->{ gas_price_cache }->{ $cur_store }->{ $HOME };
      $trans_price    = $context->{ gas_price_calc }->( $cur_store, $HOME ) unless( defined $trans_price );

      my $temp_result = $cur_store_items->{ $item } + $trans_price;
      $result = $temp_result if( $temp_result < $result );
    }
    else
    {
      my $should_go_home = ( $decide_to_go_home or ( $context->{ perishable_mask } & ( 1 << $item ) ) ) ? 1 : 0;

      my $next_store = 0;
      while( $next_store < $context->{ store_count } )
      {
        if( not ( $next_shopping_list & $context->{ stores }->[ $next_store ]->{ item_mask } ) )
        {
          $next_store++; next;
        }

        if( $next_store == $cur_store )
        {
          my $temp_result =  $cache->[ $next_shopping_list ]->[ $next_store ]->[ $should_go_home ];
          unless( defined $temp_result )
          {
            $temp_result = get_min_cost( $next_shopping_list, $next_store, $should_go_home , $context );
            $cache->[ $next_shopping_list ]->[ $next_store ]->[ $should_go_home ] = $temp_result;
          }
          $temp_result += $cur_store_items->{ $item };
          $result = $temp_result if( $temp_result < $result );
        }
        else
        {
          my $temp_result = $cache->[ $next_shopping_list ]->[ $next_store ]->[ 0 ];
          unless( defined $temp_result )
          {
            $temp_result =  get_min_cost( $next_shopping_list, $next_store, 0, $context );
            $cache->[ $next_shopping_list ]->[ $next_store ]->[ 0 ] = $temp_result;
          }

          $temp_result += $cur_store_items->{ $item };

          if( $should_go_home )
          {
            my $trans_price =  $context->{ gas_price_cache }->{ $cur_store }->{ $HOME };
            $trans_price    =  $context->{ gas_price_calc }->( $cur_store, $HOME ) unless( defined $trans_price );
            $temp_result    += $trans_price;

            $trans_price    =  $context->{ gas_price_cache }->{ $HOME }->{ $next_store };
            $trans_price    =  $context->{ gas_price_calc }->( $HOME, $next_store ) unless( defined $trans_price );
            $temp_result    += $trans_price;
          }
          else
          {
            my $trans_price =  $context->{ gas_price_cache }->{ $cur_store }->{ $next_store };
            $trans_price    =  $context->{ gas_price_calc }->( $cur_store, $next_store ) unless( defined $trans_price );
            $temp_result    += $trans_price;
          }
          $result = $temp_result if( $temp_result < $result );
        }
        $next_store++;
      }
    }
  }

  return $result;
}

sub run_case_with
{
  my ( $context ) = @_;

  my $result = INF;

  my $start_store_idx = 0;
  while( $start_store_idx < $context->{ store_count } )
  {
    my $temp_result = $context->{ gas_price_calc }->( $HOME, $start_store_idx ) + get_min_cost(
      ( 1 << $context->{ item_count } ) - 1,
      $start_store_idx, 0, $context );
    $result = $temp_result if( $temp_result < $result );
    $start_store_idx++;
  }

  return $result;
}

sub main
{
  my $total_count = <>;
  my $case_count  = 1;

  while( $case_count <= $total_count )
  {
    my ( $item_count, $store_count, $gas_price ) = split( ' ', <> );

    my ( $item_names, $perishable_mask ) = get_item_names_and_perishable_mask();
    my $stores                           = get_stores( $store_count, $item_names );

    my $context = { item_count => $item_count, store_count => $store_count, stores => $stores, perishable_mask => $perishable_mask };
    $context->{ gas_price_calc }     = get_gas_price_calc( $context, $gas_price );
    $context->{ get_min_cost_cache }->[ 1 << $item_count ] = undef;

    printf( "Case #%d: %.7f\n", $case_count++, run_case_with( $context ) );
  }
}
sub get_item_names_and_perishable_mask
{
  my $item_names      = {};
  my $perishable_mask = 0;
  my $counter = 0;
  for my $item ( split( ' ', <> ) )
  {
    if( my ( $real_name ) = ( $item =~ /(.*)!$/ ) )
    {
      $item_names->{ $real_name } = $counter++;
      $perishable_mask |= ( 1 << ( $item_names->{ $real_name } ) );
    }
    else
    {
      $item_names->{ $item } = $counter++;
    }
  }

  return ( $item_names, $perishable_mask );
}
sub get_stores
{
  my ( $store_count, $item_name_table ) = @_;

  my $result  = [];
  my $counter = 0;
  while( $counter < $store_count )
  {
    my $cur_store = $result->[ $counter++ ] = {};

    my @store_items;
    ( $cur_store->{ x }, $cur_store->{ y }, @store_items ) = split( ' ', <> );
    $cur_store->{ item_mask } = 0;
    for my $store_item ( @store_items )
    {
      my ( $store_item_name, $store_item_price ) = split( ':', $store_item );
      $cur_store->{ items }->{ $item_name_table->{ $store_item_name } } = $store_item_price;
      $cur_store->{ item_mask } |= ( 1 << $item_name_table->{ $store_item_name } );
    }
  }

  return $result;
}
sub get_gas_price_calc
{
  my ( $context, $gas_price ) = @_;

  $context->{ gas_price_cache } = {};

  return sub
  {
    my ( $store_1, $store_2 ) = @_;

    my $store_1_obj = ( $store_1 == $HOME ) ? $HOME_OBJ : $context->{ stores }->[ $store_1 ];
    my $store_2_obj = ( $store_2 == $HOME ) ? $HOME_OBJ : $context->{ stores }->[ $store_2 ];

    my $result = sqrt( ( $store_1_obj->{ x } - $store_2_obj->{ x } ) ** 2 + ( $store_1_obj->{ y } - $store_2_obj->{ y } ) ** 2 ) * $gas_price;

    $context->{ gas_price_cache }->{ $store_1 }->{ $store_2 } = $result;
    $context->{ gas_price_cache }->{ $store_2 }->{ $store_1 } = $result;

    return $result;
  }
}
main();
